/* $LAAS: evart-client.h 2009/10/01 16:20:22 mallet $ */

/* 
 * Copyright (c) 2008-2009 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Fri Jul  4 2008
 */

#ifndef H_EVART_CLIENT
#define H_EVART_CLIENT

/** \ingroup constants
 * \brief Default IP port of the evart-stream server.
 */
#define EVAS_STREAM_PORT	3615

/** \ingroup constants
 * \brief Default hostname of the evart-stream server.
 */
#define EVAS_STREAM_HOST	"shrek"

/** \ingroup constants
 * \brief Maximum length of names and miscelaneous strings parameters.
 */
#define EVAS_STRMAX	(128)

#define EVAS_EMPTY	(9999999.0f)

/** \ingroup constants
 * \brief On/off flag.
 * 
 * Boolean used for activation or deactivation of some functionality.
 */

typedef enum evas_onoff_t {
  EVAS_OFF,		/**< Deactivate. */
  EVAS_ON		/**< Activate. */
} evas_onoff_t;


/** \ingroup requests
 * \brief List of tracked bodies (a.k.a. "markersets").
 *
 * This is a variable-length structure that contains a list of currently
 * active body names. The \a bodies array contains \a nbodies strings,
 * from index 0 to \a nbodies-1.
 *
 * Use the index number of the body you are interrested in in functions
 * that require a body number. This index will not change as long as you
 * don't change the configuration of the EVaRT(tm) software.
 */

typedef struct evas_body_list_t {
  int nbodies;			/**< number of bodies. */
  char bodies[0][EVAS_STRMAX];	/**< array of body names (C strings),
				 * from 0 to \a nbodies-1. */
} evas_body_list_t;


/** \ingroup requests
 * \brief Body markers definition.
 *
 * This is a variable-length structure that contains a list of marker
 * names for a specific body. The body identity can be found in the \a
 * index and \a name fields and the \a markers array contains the list of
 * marker names, from index 0 to \a nmarkers-1.
 */

typedef struct evas_body_markers_list_t {
  int index;			/**< Index of the body. */
  char name[EVAS_STRMAX];	/**< Name of the body. */

  int nmarkers;			/**< Number of markers for that body. */
  char markers[0][EVAS_STRMAX];	/**< Array of marker names (C strings),
				 * from 0 to \a nmarkers-1. */
} evas_body_markers_list_t;


/** \ingroup data
 * \brief Marker positions for one body.
 *
 * This is a variable-length structure that contains a list of marker
 * positions for a specific body. The body identity can be found in the
 * \a index and \a name fields and the \a markers array contains the list
 * of marker positions, from index 0 to \a nmarkers-1. Each position is
 * itself an array of X,Y and Z coordinates.
 */

typedef struct evas_body_markers_t {
  int iFrame;			/**< EVaRT's frame number. */
  float fDelay;			/**< Total time (seconds) from Camera to
				 * the Host sending the data. */

  int index;			/**< Index of the body. */
  char name[EVAS_STRMAX];	/**< Name of the body. */

  int nmarkers;			/**< Number of markers for that body. */
  float markers[0][3];		/**< \brief Array of marker positions. */
				/**< markers[i][0] == EVAS_EMPTY means no
				 *  data. */
} evas_body_markers_t;


/** \ingroup data
 * \brief Unrecognized markers positions.
 *
 * This is a variable-length structure that contains a list of
 * unrecognized marker positions. The \a markers array contains the list
 * of marker positions, from index 0 to \a nmarkers-1. Each position is
 * itself an array of X,Y and Z coordinates.
 */

typedef struct evas_unrecognized_markers_t {
  int iFrame;		/**< EVaRT's frame number. */
  float fDelay;		/**< Total time (seconds) from Camera to the Host
			 * sending the data. */

  int nmarkers;		/**< Number of unrecognized markers. */
  float markers[0][3];	/**< \brief Array of marker positions. */
			/**< markers[i][0] == EVAS_EMPTY means no data. */
} evas_unrecognized_markers_t;


/** \ingroup data
 * \brief Messages types from the server.
 *
 * This is the enumeration of all possible messages types that can be
 * received the the evart-stream server.
 */

typedef enum evas_msgtype_t {
  EVAS_NONE,			/**< No message, or invalid message. */
  EVAS_MSG,			/**< Raw text message. */
  EVAS_ACQUIRE,			/**< Marker tracking. */
  EVAS_RECORD,			/**< Control data recording on the host. */
  EVAS_FPS,			/**< Get marker data frame rate. */
  EVAS_BODY_LIST,		/**< List of tracked bodies. */
  EVAS_BODY_MARKERS_LIST,	/**< List of marker names for one body. */
  EVAS_BODY_MARKERS,		/**< Marker positions for one body. */
  EVAS_UNRECOGNIZED_MARKERS	/**< Unrecognized marker positions. */
} evas_msgtype_t;


/** \ingroup data
 * \brief Messages streamed by the server.
 *
 * This structure is the basis of all data streamed by the
 * evart-server. Every function receiving data from the server will be
 * passed such messages (see evas_sethandler(), evas_recv()).
 *
 * The structure contains several fields, depending on the actual \a type
 * of the message. Use \b only the field correspondig to the \a type
 * member, as other fields will typically contain garbage.
 */

typedef struct evas_msg_t {
  unsigned int len;		/**< \brief The length of the message in
				 * bytes. */
  evas_msgtype_t type;		/**< \brief The type of the message. */
				/**< Use this field to determine validity
				 * of other members. */
  union {
    char cmd[EVAS_STRMAX];	/**< \brief For EVAS_MSG messages. */
				/**< Contains raw ASCII text from the
				 * server. This type of message is
				 * normally not used. */
    evas_body_list_t body_list;	/**< \brief For EVAS_BODY_LIST messages. */
				/**< Contains a list of body names. */

				/** \brief For EVAS_BODY_MARKERS messages. */
				/** Contains a list of marker positions. */
    evas_body_markers_t body_markers;
				/** \brief For EVAS_BODY_MARKERS_LIST
				 * messages. */
				/** Contains a list of marker names. */
    evas_body_markers_list_t body_markers_list;
				/** \brief For EVAS_UNRECOGNIZED_MARKERS
				 * messages. */
				/** Contains a list of unrecognized
				 * marker positions. */
    evas_unrecognized_markers_t unrecognized_markers;

    char raw[65536];		/**< raw undecoded message. Don't use it. */
  };
} evas_msg_t;


/* --- API ----------------------------------------------------------- */

#ifdef __cplusplus
extern "C" {
#endif

  void			evas_setport(unsigned short port);
  void			evas_sethost(const char *host);

  void			evas_sethandler(
				int (*handler)(const evas_msg_t *, void *),
				void *data);
  void			evas_listen(void);

  int			evas_acquire(evas_onoff_t on);
  int			evas_record(const char *file, evas_onoff_t on);
  int			evas_fps(float *fps);

  const evas_body_list_t *
			evas_body_list(void);
  const evas_body_markers_list_t *
			evas_body_markers_list(int body);
  int			evas_body_markers(int body, evas_onoff_t on);
  int			evas_unrecognized_markers(evas_onoff_t on);

  int			evas_text(const char *text);
  const evas_msg_t *	evas_recv(evas_msg_t *msg, double timeout);

#ifdef __cplusplus
}
#endif

#endif /* H_EVART_CLIENT */
